/*
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: cr_barrier.c,v 1.5 2003/11/14 19:55:14 phargrov Exp $
 */

#include "cr_module.h"

#include <linux/module.h>
#include <linux/wait.h>
#include <linux/sched.h>
#include <linux/spinlock.h>

void cr_barrier_init(cr_barrier_t *barrier, int count)
{

    spin_lock_init(&barrier->lock);
    init_waitqueue_head(&barrier->wait);
    barrier->count = count;
    barrier->once = 0;
    barrier->entered = 0;
    barrier->broken = 0;
}

// Called to pass a barrier, blocking for all others to do so too.
// See cr_barrier.h fo rmore information.
int cr_barrier_enter(cr_barrier_t *barrier)
{
    DECLARE_WAITQUEUE(wait, current);

    add_wait_queue(&barrier->wait, &wait);
    set_current_state(TASK_UNINTERRUPTIBLE);

    spin_lock(&barrier->lock);
    barrier->entered += 1;
    barrier->count -= 1;
    if (!barrier->count) {
	wake_up(&barrier->wait);
    } else {
	spin_unlock(&barrier->lock);
	schedule();
	spin_lock(&barrier->lock);
    }
    spin_unlock(&barrier->lock);

    current->state = TASK_RUNNING;
    remove_wait_queue(&barrier->wait, &wait);

    return !test_and_set_bit(0, &barrier->once);
}

// Called to pass a barrier w/o blocking.
// Can be called by a third party if somebody dies unexpectedly.
//
// This maintains a proper count of tasks and will wake
// those who enter it once every task has decided to either
// enter or break the barrier. 
void cr_barrier_break(cr_barrier_t *barrier)
{
    spin_lock(&barrier->lock);
    barrier->broken += 1;
    barrier->count -= 1;
    if (!barrier->count) {
	wake_up(&barrier->wait);
    }
    spin_unlock(&barrier->lock);
}
