/*
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: cr_barrier.h,v 1.5 2003/11/14 19:55:14 phargrov Exp $
 */

#ifndef _CR_BARRIER_H
#define _CR_BARRIER_H        1

#include <linux/module.h>
#include <linux/spinlock.h>
#include <linux/wait.h>

// Single-use barrier which can be "broken" (often by a "third party")
// when a task dies or wishes to pass it w/o blocking.
//
// A call to cr_barrier_enter() will not return until the barrier
// has been passed "count" times.  The term "passing the barrier"
// is taken to mean either a call to cr_barrier_enter() or to
// cr_barrier_break().
//
// Exactly one call to cr_barrier_enter will return non-zero, unless
// _all_ threads called cr_barrier_break().

typedef struct cr_barrier_s {
	spinlock_t		lock;
	wait_queue_head_t	wait;
	unsigned int		count;	// How many remain
	unsigned long		once;
	// After return from cr_barrier_enter() the 'entered' and 'broken'
	// fields detail how the tasks passed the brarrier.
	unsigned int		entered;
	unsigned int		broken;
} cr_barrier_t;

extern inline void cr_barrier_init(cr_barrier_t *barrier, int count);
extern inline int cr_barrier_enter(cr_barrier_t *barrier);
extern inline void cr_barrier_break(cr_barrier_t *barrier);

#endif
