/*
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: cr_io.c,v 1.2 2003/11/14 19:55:14 phargrov Exp $
 */

/*
 * Implements basic read and write calls for kernel space
 */

#include <asm/uaccess.h>
#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/dnotify.h>

/* If you've ever seen the code for sys_read, this might look familiar.  */
static ssize_t 
do_read(struct file *file, char * buf, size_t count)
{
    ssize_t ret;

    ret = -EBADF;
    if (file) {
        if (file->f_mode & FMODE_READ) {
            ret = locks_verify_area(FLOCK_VERIFY_READ, file->f_dentry->d_inode,
                        file, file->f_pos, count);
            if (!ret) {
                ssize_t (*read)(struct file *, char *, size_t, loff_t *);
                ret = -EINVAL;
                if (file->f_op && (read = file->f_op->read) != NULL)
                    ret = read(file, buf, count, &file->f_pos);
            }
        }
	/* We skip inode_dir_notify, since it's not exported by the kernel */
    }
    return ret;
}

static ssize_t 
do_write(struct file *file, const char * buf, size_t count)
{
    ssize_t ret;

    ret = -EBADF;
    if (file) {
        if (file->f_mode & FMODE_WRITE) {
            struct inode *inode = file->f_dentry->d_inode;
            ret = locks_verify_area(FLOCK_VERIFY_WRITE, inode, file,
                file->f_pos, count);
            if (!ret) {
                ssize_t (*write)(struct file *, const char *, size_t, loff_t *);
                ret = -EINVAL;
                if (file->f_op && (write = file->f_op->write) != NULL)
                    ret = write(file, buf, count, &file->f_pos);
            }
        }
	/* We skip inode_dir_notify, since it's not exported by the kernel */
    }
    return ret;
}

/* Basically ripped off from vmadump */

ssize_t
cr_kread(struct file *file, void *buf, size_t count)
{
    ssize_t retval;
    ssize_t bytes_read;
    ssize_t bytes_left;
    mm_segment_t oldfs;

    bytes_left = count;

    oldfs = get_fs();
    set_fs(KERNEL_DS);
    while (bytes_left) {
       bytes_read = do_read(file, buf, bytes_left);
       if (bytes_read < 0) {
	   retval = bytes_read;
	   goto out;
       } 
       if (bytes_read == 0) {
	   retval = count - bytes_left;
	   goto out;
       }
       bytes_left -= bytes_read;
       buf += bytes_read;
    }
    retval = count;
out:
    set_fs(oldfs);
    return retval; 
}

ssize_t
cr_kwrite(struct file *file, void *buf, size_t count)
{
    ssize_t retval;
    ssize_t bytes_written;
    ssize_t bytes_left;
    mm_segment_t oldfs;

    bytes_left = count;

    oldfs = get_fs();
    set_fs(KERNEL_DS);
    while (bytes_left) {
       bytes_written = do_write(file, buf, bytes_left);
       if (bytes_written < 0) {
	   retval = bytes_written;
	   goto out;
       } 
       if (bytes_written == 0) {
	   retval = count - bytes_left;
	   goto out;
       }
       bytes_left -= bytes_written;
       buf += bytes_written;
    }
    retval = count;
out:
    set_fs(oldfs);
    return retval; 
}
