/*
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: cr_module.c,v 1.10 2003/11/14 19:55:14 phargrov Exp $
 */

#include <linux/module.h>

#include <linux/init.h>

/* Find some always-exported symbols needed to validate System.map */
#include <linux/fs.h>		/* for register_{chr,blk}dev */

#include "cr_module.h"

/* XXX:  Change this to something sane... */
static const char *cr_module_version =
"$Revision: 1.10 $";

MODULE_AUTHOR("LBNL/NERSC/FTG");
MODULE_DESCRIPTION("LBNL Checkpoint/Restart kernel module");
#ifdef MODULE_LICENSE
MODULE_LICENSE("GPL");
#endif

#if CR_KERNEL_TRACING
MODULE_PARM(cr_ktrace_mask, "i");
MODULE_PARM_DESC(cr_ktrace_mask, 
		 "Mask of ORed tracing flags: see cr_ktrace.h for list");
#endif

static int __init cr_init_module(void)
{
	int err;

#if CR_KERNEL_TRACING 
	CR_INFO("Berkeley Lab Checkpoint/Restart module: %s "
		"Tracing enabled (trace_mask=0x%x)", cr_module_version, cr_ktrace_mask);
#else
	CR_INFO("Berkeley Lab Checkpoint/Restart module: %s", cr_module_version);
#endif
	/* Check current kernel against System.map used af configure time */
	if ((CR_KSYM_register_chrdev != (unsigned long)&register_chrdev) ||
	    (CR_KSYM_register_blkdev != (unsigned long)&register_blkdev)) {
		CR_ERR("Running kernel does not match the System.map used to build cr.o");
		return -EINVAL;
	}

	err = cr_proc_init();
	if (err) {
		return err;
	}

	return 0;
}

static void __exit cr_cleanup_module(void)
{
	cr_proc_cleanup();
	CR_INFO("Checkpoint/Restart module removed");
}

module_init(cr_init_module);
module_exit(cr_cleanup_module);
