/* 
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: cr_pthread.c,v 1.23 2003/11/14 19:55:24 phargrov Exp $
 */

#include <string.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <errno.h>
#include "cr_private.h"

// Indicates if the pthreads library has been linked in
int cri_pthread_present;

// Holds thread-specific data key for cri_info
pthread_key_t cri_info_key;

// Holds address of pthread_create(), which we then wrap
cri_pthread_create_t cri_real_pthread_create;

// atfork callback to reset the state
static void thread_reset(void)
{
    cri_rb_init(&cri_cs_lock);
    cri_atomic_write(&cri_live_count, 0);
}

// Wrapper for pthread_create() which doesn't do much yet
int
pthread_create(pthread_t *thread,
	       const pthread_attr_t *attr,
	       void * (*start_routine)(void *),
	       void *arg)
{
    return (*cri_real_pthread_create)(thread, attr, start_routine, arg);
}

// Interpose on fork() to ensure the pthreads version is always called
pid_t cri_fork(void)
{
  return __fork();
}
weak_alias(cri_fork,fork);


// Interpose on vfork() to ensure the pthreads version is always called
pid_t cri_vfork(void)
{
  return __vfork();
}
weak_alias(cri_vfork,vfork);

//
// Initialize pthread-dependent parts if pthreads is loaded
//
void
cri_pthread_init(void)
{
    int rc;

    // look for fork() overridden to determine if libpthread is loaded
    cri_pthread_present = (dlsym(RTLD_NEXT, "fork") ==
			   dlsym(RTLD_NEXT, "__fork"));

    // Override the pthread_create() function
    cri_real_pthread_create =
		(cri_pthread_create_t)dlsym(RTLD_NEXT, "pthread_create");
    if (cri_real_pthread_create == NULL) {
	CRI_ABORT("When linking libpthread, it must be linked AFTER libcr");
    }

    // Install a pthread_atfork callback to cleanup state in children
    rc = pthread_atfork(NULL, NULL, &thread_reset);
    if (rc != 0) {
	CRI_ABORT("pthread_atfork() returned %d", rc);
    }

    if (cri_pthread_present) {
	// Setup the thread-specific data for cri_info
	rc = pthread_key_create(&cri_info_key, &free); // XXX: better dtor?
	if (rc != 0) {
	    CRI_ABORT("pthread_key_create() returned %d", rc);
	}
    }
}
