/* 
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: cr_pthread_hack.c,v 1.20 2003/11/14 19:55:24 phargrov Exp $
 *
 *
 * THIS IS A HORRIBLE UGLY STUPID HACK THAT MANAGES TO REBUILD THE
 * PTHREAD MANAGER PIPE AND REWRITE THE PIDS IN THE PTHREAD DESCRIPTORS
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include "cr_private.h"

#if CRI_PTHREAD_HACK // OTHERWISE THIS FILE IS EMPTY

#ifndef OPEN_MAX
#  define OPEN_MAX 256
#endif

//
// Private data
//

static int		hack_pipe[2] = {-1,-1};
static cr_spinlock_t    hack_lock    = CR_SPINLOCK_INITIALIZER;
static cri_atomic_t	hack_barrier = {0,};
static cri_atomic_t	hack_once    = {0,};

//
// Private functions
//

static ino_t hack_pipe_ino(int fd)
{
    struct stat st;

    return ((fstat(fd, &st) != -1) && S_ISFIFO(st.st_mode)) ? st.st_ino : 0;
}

static void hack_find_pipe(void)
{
    // XXX: blindly assume first pair of pipes are it
    int fd1, fd2;
    ino_t ino;

    // Only need to save the pipe once for the entire life of the run
    if (!cri_cmp_swap(&hack_once, 0, 1)) {
	return;
    }

    for (fd1 = 0; fd1 < OPEN_MAX; ++fd1) {
	ino = hack_pipe_ino(fd1);
	if (ino) {
	    for (fd2 = (fd1 + 1); fd2 < OPEN_MAX; ++fd2) {
		if (ino == hack_pipe_ino(fd2)) {
		    int flags;

		    flags = fcntl(fd1, F_GETFL);
		    if (flags < 0) {
			CRI_ABORT("fcntl(pipe)");
		    } else if (flags & O_WRONLY) {
			hack_pipe[0] = fd2;
			hack_pipe[1] = fd1;
		    } else {
			hack_pipe[0] = fd1;
			hack_pipe[1] = fd2;
		    }
		    LIBCR_TRACE(LIBCR_TRACE_INFO, "[%d] found pipe at [%d,%d]",
				(int)getpid(), hack_pipe[0], hack_pipe[1]);
		    return;
		}
	    }
	}
    }
    // We failed to find the pipe.  Hopefully this is just because
    // we've not spawned the manager thread yet.  We can try again later
    LIBCR_TRACE(LIBCR_TRACE_INFO, "[%d] pthread manager pipe not located\n",
		(int)getpid());
    cri_atomic_write(&hack_once, 0);
}

// The first thread to get here will restore the pipe
// while the others wait in a barrier.
static void hack_rebuild_pipe(void)
{
    if (hack_pipe[0] < 0) {
	// Nothing to restore, no need to wait
	cri_atomic_write(&hack_barrier, 0);
	return;
    }

    if (cr_spinlock_trylock(&hack_lock)) {
        int fds[2];

        if (pipe(fds) == -1) {
	    CRI_ABORT("pipe() call failed\n");
        }

        // Move first end into place
        if (fds[0] != hack_pipe[0]) {
	    if (fds[1] == hack_pipe[0]) {
	        // move this one out of the way
	        fds[1] = dup(hack_pipe[0]);
	        close(hack_pipe[0]);
	    }
    
	    dup2(fds[0], hack_pipe[0]);
	    close(fds[0]);
        }
    
        // Move second end into place
        if (fds[1] != hack_pipe[1]) {
	    dup2(fds[1], hack_pipe[1]);
	    close(fds[1]);
        }
    
        LIBCR_TRACE(LIBCR_TRACE_INFO, "[%d] restored pipe at [%d,%d]",
		    (int)getpid(), hack_pipe[0], hack_pipe[1]);

	(void)cri_barrier_enter(&hack_barrier);
	cr_spinlock_unlock(&hack_lock);
    } else {
	(void)cri_barrier_enter(&hack_barrier);
    }
}

//
// Internal functions
//

void cri_pthread_hack_checkpoint(void)
{
    if (cri_pthread_present) {
	hack_find_pipe();
	cri_atomic_inc(&hack_barrier);
    }
}

void cri_pthread_hack_continue(void)
{
    cri_atomic_write(&hack_barrier, 0);
}

void cri_pthread_hack_restart(void)
{
    if (cri_pthread_present) {
	hack_rebuild_pipe();
    }
}

#endif /* CRI_PTHREAD_HACK */
