/*
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: cr_test.c,v 1.4 2003/11/14 19:55:33 phargrov Exp $
 */


const char description[] = 
"Description of tests/FailedCB/cr_test:\n"
"\n"
" Verifies that when a callback invokes cr_checkpoint() with a non-zero\n"
" argument, additional callbacks will be skipped/\n"
"\n"
"In a successful run, expect lines of output in SORTED order.\n"
;


#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

#include "libcr.h"

static int callback0(void* arg)
{
    int rc;

    printf("005 enter callback\n");
    rc = cr_checkpoint(CR_CHECKPOINT_TEMP_FAILURE);
    if (rc >= 0) {
	printf("XXX resuming unexpectedly in callback\n");
	exit(-1);
    } else if (rc != -CR_ETEMPFAIL) {
	printf("XXX checkpoint TEMP cancellation failed: %d (%s)\n", rc, cr_strerror(-rc));
	exit(-1);
    } else {
	printf("006 checkpoint cancelled successfully with TEMP fail\n");
    }

    return 0;
}

static int callback1(void* arg)
{
    printf("XXX enter 'other' callback unexpectedly\n");
    return 0;
}

int main(void)
{
    pid_t my_pid;
    cr_callback_id_t cb_id;
    cr_client_id_t my_id;
    char filename[40];
    int rc;
    struct stat s;

    setlinebuf(stdout);
	
    my_pid = getpid();
    snprintf(filename, sizeof(filename), "context.%d", my_pid);
    printf("000 Process started with pid %d\n", my_pid);

    my_id = cr_init();
    if (my_id < 0) {
	printf("XXX cr_init() failed, returning %d\n", rc);
	exit(-1);
    } else {
	printf("001 cr_init() succeeded\n");
    }

    cb_id = cr_register_callback(callback1, NULL, CR_SIGNAL_CONTEXT);
    if (cb_id < 0) {
	printf("XXX cr_register_callback() unexpectedly returned %d\n", cb_id);
	exit(-1);
    } else {
	printf("002 cr_register_callback() correctly returned %d\n", cb_id);
    }

    cb_id = cr_register_callback(callback0, NULL, CR_SIGNAL_CONTEXT);
    if (cb_id < 0) {
	printf("XXX cr_register_callback() unexpectedly returned %d\n", cb_id);
	exit(-1);
    } else {
	printf("003 cr_register_callback() correctly returned %d\n", cb_id);
    }

    rc = cr_status();
    if (rc != CR_STATE_IDLE) {
	printf("XXX cr_status() unexpectedly returned %d\n", rc);
	exit(-1);
    } else {
	printf("004 cr_status() correctly returned 0x%x\n", rc);
    }

    cr_request();

    rc = cr_status();
    if (rc != CR_STATE_IDLE) {
	printf("XXX cr_status() unexpectedly returned 0x%x\n", rc);
	exit(-1);
    } else {
	printf("007 cr_status() correctly returned 0x%x\n", rc);
    }

    rc = stat(filename, &s);
    if (!rc) {
	printf("XXX stat() unexpectedly succeeded\n", rc);
	exit(-1);
    } else {
	printf("008 stat(context.%d) correctly failed\n", my_pid);
    }

    (void)unlink(filename); // might fail silently

    printf("009 DONE\n");

    return 0;
}
