/*
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: cr_test.c,v 1.6 2003/11/14 19:55:35 phargrov Exp $
 */


#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "libcr.h"

static int cb0(void* arg);
static int cb1(void* arg);
static int cb2(void* arg);
static int cb3(void* arg);

int count = 0;

static void out(const char* s)
{
    write(1, s, strlen(s));
}

static int cb0(void* arg)
{
    out(arg); out(" in cb0\n");
    cr_replace_self(cb2, "003", CR_THREAD_CONTEXT);
    return 0;
}

static int cb1(void* arg)
{
    out(arg); out(" in cb1\n");
    cr_replace_self(cb3, "004", CR_SIGNAL_CONTEXT);
    return 0;
}

static int cb2(void* arg)
{
    out(arg); out(" in cb2\n");
    return 0;
}

static int cb3(void* arg)
{
    out(arg); out(" in cb3\n");
    return 0;
}

int main(void)
{
    pid_t my_pid;
    cr_callback_id_t id0;
    cr_callback_id_t id1;
    cr_client_id_t my_id;
    char filename[40];
	
    setlinebuf(stdout);

    my_pid = getpid();
    snprintf(filename, sizeof(filename), "context.%d", my_pid);
    printf("000 Process started with pid %d\n", my_pid);
    fflush(stdout);

    my_id = cr_init();

    id0 = cr_register_callback(cb0, "001", CR_THREAD_CONTEXT);
    id1 = cr_register_callback(cb1, "002", CR_SIGNAL_CONTEXT);

    cr_request();
    
    cr_request();

    cr_enter_cs(my_id);
    cr_replace_callback(id0, cb0, "005", CR_THREAD_CONTEXT);
    cr_replace_callback(id1, cb1, "006", CR_SIGNAL_CONTEXT);
    cr_replace_callback(id0, cb2, "XXX", CR_SIGNAL_CONTEXT); // should fail
    cr_replace_callback(id1, cb3, "XXX", CR_THREAD_CONTEXT); // should fail
    cr_leave_cs(my_id);

    cr_request();

    cr_enter_cs(my_id);
    cr_leave_cs(my_id);

    (void)unlink(filename);	// may fail silently

    out("007 DONE\n");

    return 0;
}
